@@ -52,10 +52,31 @@ KERNAL_FILE_SAMPLE_V6 = (
     "  -> [fd79:35e2::8f3f]:115C       "
     "Masq    2      0          0")
 
+KERNEL_FILE_SAMPLE_MIXED = (
+    "IP Virtual Server version 1.2.1 (size=4096)\n"
+    "Prot LocalAddress:Port Scheduler Flags\n"
+    "  -> RemoteAddress:Port Forward Weight ActiveConn InActConn\n"
+    "UDP  [fd79:35e2:9963:0000:f816:3eff:fe6d:7a2a]:1E61 rr\n"
+    "  -> [fd79:35e2:9963:0000:f816:3eff:feca:b7bf]:08AE      "
+    "Masq    3      0          0\n"
+    "  -> [fd79:35e2:9963:0000:f816:3eff:fe9d:94df]:0D05      "
+    "Masq    2      0          0\n"
+    "  -> [fd79:35e2::8f3f]:115C       "
+    "Masq    2      0          0\n"
+    "UDP  0A000025:1E61 rr\n"
+    "  -> 0A000023:0D05      Masq    2      0          0\n"
+    "  -> 0A000019:08AE      Masq    3      0          0\n"
+    "UDP  0A000025:FF12 rr\n"
+    "  -> 0A000023:12AB      Masq    2      0          0\n"
+    "  -> 0A000019:BF2E      Masq    3      0          0")
+
 CFG_FILE_TEMPLATE_v4 = (
     "# Configuration for Listener %(listener_id)s\n\n"
     "net_namespace %(ns_name)s\n\n"
-    "virtual_server 10.0.0.37 7777 {\n"
+    "virtual_server_group ipv4-group {\n"
+    "   10.0.0.37 7777\n"
+    "}\n\n"
+    "virtual_server group ipv4-group {\n"
     "    lb_algo rr\n"
     "    lb_kind NAT\n"
     "    protocol udp\n\n\n"
@@ -96,7 +117,10 @@ CFG_FILE_TEMPLATE_v4 = (
 CFG_FILE_TEMPLATE_v6 = (
     "# Configuration for Listener %(listener_id)s\n\n"
     "net_namespace %(ns_name)s\n\n"
-    "virtual_server fd79:35e2:9963:0:f816:3eff:fe6d:7a2a 7777 {\n"
+    "virtual_server_group ipv6-group {\n"
+    "   fd79:35e2:9963:0:f816:3eff:fe6d:7a2a 7777\n"
+    "}\n\n"
+    "virtual_server group ipv6-group {\n"
     "    lb_algo rr\n"
     "    lb_kind NAT\n"
     "    protocol udp\n\n\n"
@@ -136,6 +160,85 @@ CFG_FILE_TEMPLATE_v6 = (
     "    }\n\n"
     "}")
 
+CFG_FILE_TEMPLATE_mixed = (
+    "# Configuration for Listener %(listener_id)s\n\n"
+    "net_namespace %(ns_name)s\n\n"
+    "virtual_server_group ipv4-group {\n"
+    "   10.0.0.37 7777\n"
+    "}\n\n"
+    "virtual_server group ipv4-group {\n"
+    "    lb_algo rr\n"
+    "    lb_kind NAT\n"
+    "    protocol udp\n\n\n"
+    "    # Configuration for Pool %(pool_id)s\n"
+    "    # Configuration for Member %(member_id1)s\n"
+    "    real_server 10.0.0.25 2222 {\n"
+    "        weight 3\n"
+    "        MISC_CHECK {\n\n"
+    "            misc_path \"/usr/bin/check_script.sh\"\n\n"
+    "            misc_timeout 5\n\n"
+    "        }\n\n"
+    "    }\n\n"
+    "    # Member %(member_id2)s is disabled\n\n"
+    "}\n"
+    "virtual_server_group ipv6-group {\n"
+    "   fd79:35e2:9963:0:f816:3eff:fe6d:7a2a 7777\n"
+    "}\n\n"
+    "virtual_server group ipv6-group {\n"
+    "    lb_algo rr\n"
+    "    lb_kind NAT\n"
+    "    protocol udp\n\n\n"
+    "    # Configuration for Pool %(pool_id)s\n"
+    "    # Configuration for Member %(member_id3)s\n"
+    "    real_server fd79:35e2:9963:0:f816:3eff:feca:b7bf 2222 {\n"
+    "        weight 3\n"
+    "        MISC_CHECK {\n\n"
+    "            misc_path \"/usr/bin/check_script.sh\"\n\n"
+    "            misc_timeout 5\n\n"
+    "        }\n\n"
+    "    }\n\n"
+    "    # Configuration for Member %(member_id4)s\n"
+    "    real_server fd79:35e2:9963:0:f816:3eff:fe9d:94df 3333 {\n"
+    "        weight 2\n"
+    "        MISC_CHECK {\n\n"
+    "            misc_path \"/usr/bin/check_script.sh\"\n\n"
+    "            misc_timeout 5\n\n"
+    "        }\n\n"
+    "    }\n\n"
+    "    # Member %(member_id5)s is disabled\n\n"
+    "}")
+
+CFG_FILE_TEMPLATE_mixed_no_ipv6_member = (
+    "# Configuration for Listener %(listener_id)s\n\n"
+    "net_namespace %(ns_name)s\n\n"
+    "virtual_server_group ipv4-group {\n"
+    "   10.0.0.37 7777\n"
+    "}\n\n"
+    "virtual_server group ipv4-group {\n"
+    "    lb_algo rr\n"
+    "    lvs_method NAT\n"
+    "    protocol udp\n\n\n"
+    "    # Configuration for Pool %(pool_id)s\n"
+    "    # Configuration for Member %(member_id1)s\n"
+    "    real_server 10.0.0.25 2222 {\n"
+    "        weight 3\n"
+    "        MISC_CHECK {\n\n"
+    "            misc_path \"/usr/bin/check_script.sh\"\n\n"
+    "            misc_timeout 5\n\n"
+    "        }\n\n"
+    "    }\n\n"
+    "    # Member %(member_id2)s is disabled\n\n"
+    "}\n"
+    "virtual_server_group ipv6-group {\n"
+    "   fd79:35e2:9963:0:f816:3eff:fe6d:7a2a 7777\n"
+    "}\n\n"
+    "virtual_server group ipv6-group {\n"
+    "    lb_algo rr\n"
+    "    lvs_method NAT\n"
+    "    protocol udp\n\n\n"
+    "    # Configuration for Pool %(pool_id)s\n"
+    "}")
+
 CFG_FILE_TEMPLATE_DISABLED_LISTENER = (
     "# Listener %(listener_id)s is disabled \n\n"
     "net_namespace %(ns_name)s\n\n"
@@ -178,6 +281,9 @@ class LvsQueryTestCase(base.TestCase):
         self.member_id3_v6 = uuidutils.generate_uuid()
         self.member_id4_v6 = uuidutils.generate_uuid()
         self.member_id5_v6 = uuidutils.generate_uuid()
+        self.listener_id_mixed = uuidutils.generate_uuid()
+        self.listener_id_mixed_no_ipv6_member = uuidutils.generate_uuid()
+        self.pool_id_mixed = uuidutils.generate_uuid()
         self.disabled_listener_id = uuidutils.generate_uuid()
         cfg_content_v4 = CFG_FILE_TEMPLATE_v4 % {
             'listener_id': self.listener_id_v4,
@@ -198,6 +304,25 @@ class LvsQueryTestCase(base.TestCase):
             'member_id4': self.member_id4_v6,
             'member_id5': self.member_id5_v6
         }
+        cfg_content_mixed = CFG_FILE_TEMPLATE_mixed % {
+            'listener_id': self.listener_id_mixed,
+            'ns_name': constants.AMPHORA_NAMESPACE,
+            'pool_id': self.pool_id_mixed,
+            'member_id1': self.member_id1_v4,
+            'member_id2': self.member_id2_v4,
+            'member_id3': self.member_id3_v6,
+            'member_id4': self.member_id4_v6,
+            'member_id5': self.member_id5_v6
+        }
+        cfg_content_mixed_no_ipv6_member = (
+            CFG_FILE_TEMPLATE_mixed_no_ipv6_member % {
+                'listener_id': self.listener_id_mixed,
+                'ns_name': constants.AMPHORA_NAMESPACE,
+                'pool_id': self.pool_id_mixed,
+                'member_id1': self.member_id1_v4,
+                'member_id2': self.member_id2_v4
+            }
+        )
         cfg_content_disabled_listener = (
             CFG_FILE_TEMPLATE_DISABLED_LISTENER % {
                 'listener_id': self.listener_id_v6,
@@ -208,6 +333,13 @@ class LvsQueryTestCase(base.TestCase):
             util.keepalived_lvs_cfg_path(self.listener_id_v4), cfg_content_v4))
         self.useFixture(test_utils.OpenFixture(
             util.keepalived_lvs_cfg_path(self.listener_id_v6), cfg_content_v6))
+        self.useFixture(test_utils.OpenFixture(
+            util.keepalived_lvs_cfg_path(self.listener_id_mixed),
+            cfg_content_mixed))
+        self.useFixture(test_utils.OpenFixture(
+            util.keepalived_lvs_cfg_path(
+                self.listener_id_mixed_no_ipv6_member),
+            cfg_content_mixed_no_ipv6_member))
         self.useFixture(test_utils.OpenFixture(
             util.keepalived_lvs_cfg_path(self.disabled_listener_id),
             cfg_content_disabled_listener))
@@ -215,7 +347,7 @@ class LvsQueryTestCase(base.TestCase):
     @mock.patch('subprocess.check_output')
     def test_get_listener_realserver_mapping(self, mock_check_output):
         # Ipv4 resolver
-        input_listener_ip_port = '10.0.0.37:7777'
+        input_listener_ip_port = ['10.0.0.37:7777']
         target_ns = constants.AMPHORA_NAMESPACE
         mock_check_output.return_value = KERNAL_FILE_SAMPLE_V4
         result = lvs_query.get_listener_realserver_mapping(
@@ -231,10 +363,11 @@ class LvsQueryTestCase(base.TestCase):
                                        'Weight': '2',
                                        'ActiveConn': '0',
                                        'InActConn': '0'}}
-        self.assertEqual((True, expected), result)
+        self.assertEqual(expected, result)
 
         # Ipv6 resolver
-        input_listener_ip_port = '[fd79:35e2:9963:0:f816:3eff:fe6d:7a2a]:7777'
+        input_listener_ip_port = [
+            '[fd79:35e2:9963:0:f816:3eff:fe6d:7a2a]:7777']
         mock_check_output.return_value = KERNAL_FILE_SAMPLE_V6
         result = lvs_query.get_listener_realserver_mapping(
             target_ns, input_listener_ip_port,
@@ -257,31 +390,69 @@ class LvsQueryTestCase(base.TestCase):
                          'Weight': '2',
                          'ActiveConn': '0',
                          'InActConn': '0'}}
-        self.assertEqual((True, expected), result)
+        self.assertEqual(expected, result)
+
+        # mixed resolver
+        input_listener_ip_port = [
+            '[fd79:35e2:9963:0:f816:3eff:fe6d:7a2a]:7777',
+            '10.0.0.37:7777']
+        mock_check_output.return_value = KERNEL_FILE_SAMPLE_MIXED
+        result = lvs_query.get_listener_realserver_mapping(
+            target_ns, input_listener_ip_port,
+            health_monitor_enabled=True)
+        expected = {'[fd79:35e2:9963:0:f816:3eff:feca:b7bf]:2222':
+                    {'status': constants.UP,
+                     'Forward': 'Masq',
+                     'Weight': '3',
+                     'ActiveConn': '0',
+                     'InActConn': '0'},
+                    '[fd79:35e2:9963:0:f816:3eff:fe9d:94df]:3333':
+                        {'status': constants.UP,
+                         'Forward': 'Masq',
+                         'Weight': '2',
+                         'ActiveConn': '0',
+                         'InActConn': '0'},
+                    '[fd79:35e2::8f3f]:4444':
+                        {'status': constants.UP,
+                         'Forward': 'Masq',
+                         'Weight': '2',
+                         'ActiveConn': '0',
+                         'InActConn': '0'},
+                    '10.0.0.25:2222': {'status': 'UP',
+                                       'Forward': 'Masq',
+                                       'Weight': '3',
+                                       'ActiveConn': '0',
+                                       'InActConn': '0'},
+                    '10.0.0.35:3333': {'status': 'UP',
+                                       'Forward': 'Masq',
+                                       'Weight': '2',
+                                       'ActiveConn': '0',
+                                       'InActConn': '0'}}
+        self.assertEqual(expected, result)
 
         # negetive cases
         mock_check_output.return_value = KERNAL_FILE_SAMPLE_V4
         for listener_ip_port in ['10.0.0.37:7776', '10.0.0.31:7777']:
             result = lvs_query.get_listener_realserver_mapping(
-                target_ns, listener_ip_port,
+                target_ns, [listener_ip_port],
                 health_monitor_enabled=True)
-            self.assertEqual((False, {}), result)
+            self.assertEqual({}, result)
 
         mock_check_output.return_value = KERNAL_FILE_SAMPLE_V6
         for listener_ip_port in [
             '[fd79:35e2:9963:0:f816:3eff:fe6d:7a2a]:7776',
                 '[fd79:35e2:9973:0:f816:3eff:fe6d:7a2a]:7777']:
             result = lvs_query.get_listener_realserver_mapping(
-                target_ns, listener_ip_port,
+                target_ns, [listener_ip_port],
                 health_monitor_enabled=True)
-            self.assertEqual((False, {}), result)
+            self.assertEqual({}, result)
 
     def test_get_lvs_listener_resource_ipports_nsname(self):
         # ipv4
         res = lvs_query.get_lvs_listener_resource_ipports_nsname(
             self.listener_id_v4)
         expected = {'Listener': {'id': self.listener_id_v4,
-                                 'ipport': '10.0.0.37:7777'},
+                                 'ipports': ['10.0.0.37:7777']},
                     'Pool': {'id': self.pool_id_v4},
                     'Members': [{'id': self.member_id1_v4,
                                  'ipport': '10.0.0.25:2222'},
@@ -298,7 +469,7 @@ class LvsQueryTestCase(base.TestCase):
             self.listener_id_v6)
         expected = {'Listener': {
             'id': self.listener_id_v6,
-            'ipport': '[fd79:35e2:9963:0:f816:3eff:fe6d:7a2a]:7777'},
+            'ipports': ['[fd79:35e2:9963:0:f816:3eff:fe6d:7a2a]:7777']},
             'Pool': {'id': self.pool_id_v6},
             'Members': [
                 {'id': self.member_id1_v6,
@@ -318,6 +489,28 @@ class LvsQueryTestCase(base.TestCase):
             self.disabled_listener_id)
         self.assertEqual((None, constants.AMPHORA_NAMESPACE), res)
 
+        # multi-vip/mixed
+        res = lvs_query.get_lvs_listener_resource_ipports_nsname(
+            self.listener_id_mixed)
+        expected = {'Listener': {
+            'id': self.listener_id_mixed,
+            'ipports': [
+                '10.0.0.37:7777',
+                '[fd79:35e2:9963:0:f816:3eff:fe6d:7a2a]:7777']},
+            'Pool': {'id': self.pool_id_mixed},
+            'Members': [
+            {'id': self.member_id1_v4,
+             'ipport': '10.0.0.25:2222'},
+            {'id': self.member_id3_v6,
+             'ipport': '[fd79:35e2:9963:0:f816:3eff:feca:b7bf]:2222'},
+            {'id': self.member_id4_v6,
+             'ipport': '[fd79:35e2:9963:0:f816:3eff:fe9d:94df]:3333'},
+            {'id': self.member_id2_v4,
+             'ipport': None},
+            {'id': self.member_id5_v6,
+             'ipport': None}]}
+        self.assertEqual((expected, constants.AMPHORA_NAMESPACE), res)
+
     @mock.patch('os.stat')
     @mock.patch('subprocess.check_output')
     def test_get_lvs_listener_pool_status(self, mock_check_output,
@@ -393,7 +586,7 @@ class LvsQueryTestCase(base.TestCase):
             {
                 'Listener': {
                     'id': self.listener_id_v4,
-                    'ipport': '10.0.0.37:7777'}},
+                    'ipports': ['10.0.0.37:7777']}},
             constants.AMPHORA_NAMESPACE)
         res = lvs_query.get_lvs_listener_pool_status(self.listener_id_v4)
         self.assertEqual({}, res)
@@ -410,7 +603,7 @@ class LvsQueryTestCase(base.TestCase):
         mock_get_resource_ipports.return_value = (
             {
                 'Listener': {'id': self.listener_id_v4,
-                             'ipport': '10.0.0.37:7777'},
+                             'ipports': ['10.0.0.37:7777']},
                 'Pool': {'id': self.pool_id_v4}},
             constants.AMPHORA_NAMESPACE)
         res = lvs_query.get_lvs_listener_pool_status(self.listener_id_v4)
@@ -434,7 +627,7 @@ class LvsQueryTestCase(base.TestCase):
             mock.Mock(st_mtime=1234),
             mock.Mock(st_mtime=1234),
         )
-        mock_get_mapping.return_value = (False, {})
+        mock_get_mapping.return_value = {}
         res = lvs_query.get_lvs_listener_pool_status(self.listener_id_v4)
         expected = {
             'lvs':
@@ -537,6 +730,55 @@ class LvsQueryTestCase(base.TestCase):
         res = lvs_query.get_lvs_listeners_stats()
         self.assertEqual({}, res)
 
+        # listener for both ipv4 and ipv6, but no member in the ipv6 pool
+        mock_is_running.return_value = True
+        mock_get_listener.return_value = [
+            self.listener_id_mixed_no_ipv6_member]
+        output_list = list()
+        output_list.append(IPVSADM_OUTPUT_TEMPLATE % {
+            "listener_ipport": "10.0.0.37:7777",
+            "member1_ipport": "10.0.0.25:2222",
+            "member2_ipport": "10.0.0.35:3333"})
+        output_list.append(IPVSADM_STATS_OUTPUT_TEMPLATE % {
+            "listener_ipport": "10.0.0.37:7777",
+            "member1_ipport": "10.0.0.25:2222",
+            "member2_ipport": "10.0.0.35:3333"})
+        mock_check_output.side_effect = output_list
+        res = lvs_query.get_lvs_listeners_stats()
+        # We can check the expected result reference the stats sample,
+        # that means this func can compute the stats info of single listener.
+        expected = {self.listener_id_mixed_no_ipv6_member: {
+            'status': constants.OPEN,
+            'stats': {'bin': 6387472, 'stot': 5, 'bout': 7490,
+                      'ereq': 0, 'scur': 0}}}
+        self.assertEqual(expected, res)
+
+    @mock.patch('subprocess.check_output')
+    @mock.patch("octavia.amphorae.backends.agent.api_server.util."
+                "is_lvs_listener_running", return_value=True)
+    @mock.patch("octavia.amphorae.backends.agent.api_server.util."
+                "get_lvs_listeners")
+    def test_get_lvs_listeners_stats_missing_listener(
+            self, mock_get_listener, mock_is_running, mock_check_output):
+        # The ipv6 test is same with ipv4, so just test ipv4 here
+        mock_get_listener.return_value = [self.listener_id_v4]
+        output_list = list()
+        output_list.append(IPVSADM_OUTPUT_TEMPLATE % {
+            "listener_ipport": "10.0.0.37:7778",
+            "member1_ipport": "10.0.0.25:2222",
+            "member2_ipport": "10.0.0.35:3333"})
+        output_list.append(IPVSADM_STATS_OUTPUT_TEMPLATE % {
+            "listener_ipport": "10.0.0.37:7778",
+            "member1_ipport": "10.0.0.25:2222",
+            "member2_ipport": "10.0.0.35:3333"})
+        mock_check_output.side_effect = output_list
+        res = lvs_query.get_lvs_listeners_stats()
+        expected = {self.listener_id_v4: {
+            'status': constants.OPEN,
+            'stats': {'bin': 0, 'stot': 0, 'bout': 0,
+                      'ereq': 0, 'scur': 0}}}
+        self.assertEqual(expected, res)
+
     @mock.patch('subprocess.check_output')
     @mock.patch("octavia.amphorae.backends.agent.api_server.util."
                 "is_lvs_listener_running", return_value=True)
