@@ -686,7 +686,6 @@ class TestHaproxyCfg(base.TestCase):
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
               "    timeout check 31s\n"
-              "    option ssl-hello-chk\n"
               "    fullconn {maxconn}\n"
               "    option allbackups\n"
               "    timeout connect 5000\n"
@@ -922,7 +921,7 @@ class TestHaproxyCfg(base.TestCase):
         be = ("backend sample_pool_id_1:sample_listener_id_1\n"
               "    mode http\n"
               "    balance roundrobin\n"
-              "    stick-table type ip size 10k\n"
+              "    stick-table type ipv6 size 10k\n"
               "    stick on src\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
@@ -1204,11 +1203,11 @@ class TestHaproxyCfg(base.TestCase):
               "    timeout connect 5000\n"
               "    timeout server 50000\n"
               "    server sample_member_id_1 10.0.0.99:82 weight 13 "
-              "check inter 30s fall 3 rise 2 cookie sample_member_id_1 "
-              "{opts} alpn {alpn}\n"
+              "check check-alpn {alpn} inter 30s fall 3 rise 2 cookie "
+              "sample_member_id_1 {opts} alpn {alpn}\n"
               "    server sample_member_id_2 10.0.0.98:82 weight 13 "
-              "check inter 30s fall 3 rise 2 cookie sample_member_id_2 "
-              "{opts} alpn {alpn}\n\n").format(
+              "check check-alpn {alpn} inter 30s fall 3 rise 2 cookie "
+              "sample_member_id_2 {opts} alpn {alpn}\n\n").format(
             maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
             opts="ssl crt %s verify none sni ssl_fc_sni" % cert_file_path +
                  " ciphers " + constants.CIPHERS_OWASP_SUITE_B +
@@ -1283,11 +1282,11 @@ class TestHaproxyCfg(base.TestCase):
               "    timeout connect 5000\n"
               "    timeout server 50000\n"
               "    server sample_member_id_1 10.0.0.99:82 weight 13 "
-              "check inter 30s fall 3 rise 2 cookie sample_member_id_1 "
-              "{opts} alpn {alpn}\n"
+              "check check-alpn {alpn} inter 30s fall 3 rise 2 cookie "
+              "sample_member_id_1 {opts} alpn {alpn}\n"
               "    server sample_member_id_2 10.0.0.98:82 weight 13 "
-              "check inter 30s fall 3 rise 2 cookie sample_member_id_2 "
-              "{opts} alpn {alpn}\n\n").format(
+              "check check-alpn {alpn} inter 30s fall 3 rise 2 cookie "
+              "sample_member_id_2 {opts} alpn {alpn}\n\n").format(
             maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
             opts="ssl crt %s verify none sni ssl_fc_sni" % cert_file_path +
                  " ciphers " + constants.CIPHERS_OWASP_SUITE_B,
@@ -1323,11 +1322,11 @@ class TestHaproxyCfg(base.TestCase):
               "    timeout connect 5000\n"
               "    timeout server 50000\n"
               "    server sample_member_id_1 10.0.0.99:82 weight 13 "
-              "check inter 30s fall 3 rise 2 cookie sample_member_id_1 "
-              "{opts} alpn {alpn}\n"
+              "check check-alpn {alpn} inter 30s fall 3 rise 2 cookie "
+              "sample_member_id_1 {opts} alpn {alpn}\n"
               "    server sample_member_id_2 10.0.0.98:82 weight 13 "
-              "check inter 30s fall 3 rise 2 cookie sample_member_id_2 "
-              "{opts} alpn {alpn}\n\n").format(
+              "check check-alpn {alpn} inter 30s fall 3 rise 2 cookie "
+              "sample_member_id_2 {opts} alpn {alpn}\n\n").format(
             maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
             opts="ssl crt %s verify none sni ssl_fc_sni" % cert_file_path +
                  " no-sslv3 no-tlsv10 no-tlsv11",
@@ -1423,11 +1422,11 @@ class TestHaproxyCfg(base.TestCase):
               "    timeout connect 5000\n"
               "    timeout server 50000\n"
               "    server sample_member_id_1 10.0.0.99:82 weight 13 "
-              "check inter 30s fall 3 rise 2 cookie sample_member_id_1 "
-              "{opts} alpn {alpn}\n"
+              "check check-alpn {alpn} inter 30s fall 3 rise 2 cookie "
+              "sample_member_id_1 {opts} alpn {alpn}\n"
               "    server sample_member_id_2 10.0.0.98:82 weight 13 "
-              "check inter 30s fall 3 rise 2 cookie sample_member_id_2 "
-              "{opts} alpn {alpn}\n\n").format(
+              "check check-alpn {alpn} inter 30s fall 3 rise 2 cookie "
+              "sample_member_id_2 {opts} alpn {alpn}\n\n").format(
             maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
             opts="%s %s %s %s %s %s" % (
                 "ssl", "crt", pool_client_cert,
@@ -1559,6 +1558,28 @@ class TestHaproxyCfg(base.TestCase):
         self.assertLess(ret['global_connection_limit'],
                         connection_limit_sum)
 
+    def test_transform_with_disabled_listeners(self):
+        in_amphora = sample_configs_combined.sample_amphora_tuple()
+
+        in_listeners = []
+
+        connection_limit_sum = 0
+
+        in_listener = (
+            sample_configs_combined.sample_listener_tuple())
+        connection_limit_sum += constants.HAPROXY_DEFAULT_MAXCONN
+        in_listeners.append(in_listener)
+
+        disabled_listener = (
+            sample_configs_combined.sample_listener_tuple(enabled=False))
+        in_listeners.append(disabled_listener)
+
+        ret = self.jinja_cfg._transform_loadbalancer(
+            in_amphora, in_listeners[0].load_balancer,
+            in_listeners, None, {})
+        self.assertEqual(ret['global_connection_limit'],
+                         connection_limit_sum)
+
     def test_transform_amphora(self):
         in_amphora = sample_configs_combined.sample_amphora_tuple()
         ret = self.jinja_cfg._transform_amphora(in_amphora, {})
